```jsx
<Table 
  title={'Table'}
  data={[
    { id: 'row1Id', col1: 'row 1 col 1', col2: 'row 1 col 2', col3: 'row 1 col 3', col4: 'row 1 col 4', col5: 'row 1 col 5', col6: 'row 1 col 6', col7: 'row 1 col 7', col8: 'row 1 col 8', col9: 'row 1 col 9', col10: 'row 1 col 10' },      
    { id: 'row2Id', col1: 'row 2 col 1', col2: 'row 2 col 2', col3: 'row 2 col 3', col4: 'row 2 col 4', col5: 'row 2 col 5', col6: 'row 2 col 6', col7: 'row 2 col 7', col8: 'row 2 col 8', col9: 'row 2 col 9', col10: 'row 1 col 10' },  
  ]}
  columns={[
    { dataField: 'col1', text: 'column 1', sort: true }, 
    { dataField: 'col2', text: 'column 2', sort: true },      
    { dataField: 'col3', text: 'column 3', sort: true }, 
    { dataField: 'col4', text: 'column 4 with long title', sort: true },
    { dataField: 'col5', text: 'column 5', minWidth: 200, maxWidth: 250, sort: true }, 
    { dataField: 'col6', text: 'column 6', minWidth: 200, maxWidth: 250 },
    { dataField: 'col7', text: 'column 7' }, 
    { dataField: 'col8', text: 'column 8' },
    { dataField: 'col9', text: 'column 9', sort: true }, 
    { dataField: 'col10', text: 'column 10', sort: true },
  ]}
/>
```

### Externally Controlled Table
Use the `remote` prop to control table functionality externally

onTableChange callback function gives current state of the table on all change vents

#### Available onTableChange props:
```json
{
  page,  // newest page
  sizePerPage,  // newest sizePerPage
  sortField,  // newest sort field
  sortOrder,  // newest sort order
  filters, // an object which have current filter status per column
  data, // when you enable remote sort, you may need to base on data to sort if data is filtered/searched
  cellEdit: {  // You can only see this prop when type is cellEdit
    rowId,
    dataField,
    newValue
  }
}
```
```jsx
import { Async } from 'state-template';

class ExampleTable extends React.Component {
  constructor(props) {
    super(props);
    this.state = { 
      page: 1,
      sizePerPage: 5,
      currentData: [],
      data: [],
      isLoading: false,
    };
    // binding due to compilier not being able to use arrow functions...
    this.getInitialData = this.getInitialData.bind(this);
    this.onTableChange = this.onTableChange.bind(this);
  }
  componentDidMount() {
    this.getInitialData() 
  }

  getInitialData() {
    const { page, sizePerPage } = this.state
    const currentIndex = (page - 1) * sizePerPage;
    const data = Array(300).fill(0).map((x, i) => ({
      id: i,
      col1: `row ${i} col 1`,
      col2: `row ${i} col 2`,
    }));

    const currentData = data.slice(currentIndex, currentIndex + sizePerPage)

    this.setState(state => ({
      currentData,
      data
    }))
  }

  // Callback used for all table changes
  onTableChange(type, { page, sizePerPage }) {
    const { data } = this.state;
    const currentIndex = (page - 1) * sizePerPage;
    this.setState(() => ({
      isLoading: true
    }));
    setTimeout(() => {
      this.setState(() => ({
        isLoading: false,
        page,
        currentData: data.slice(currentIndex, currentIndex + sizePerPage),
        sizePerPage
      }));
    }, 2000);
  }

  render () {
    const { data, currentData, page, sizePerPage, isLoading } = this.state

    return (
      <Async isLoading={isLoading}>
        <Table 
          title={ 'Externally Controlled Table' }
          hideSearch
          remote 
          data={ currentData }
          page={ page }
          sizePerPage={ sizePerPage }
          totalSize={ data.length }
          onTableChange={ this.onTableChange }
          columns={[
            { dataField: 'col1', text: 'column 1', sort: true }, 
            { dataField: 'col2', text: 'column 2' },
          ]}
        />
      </Async>
    )
  }
}

<ExampleTable />

```

### Client Side Pagination

```jsx
const aLotOfData = Array(300).fill(0).map((x, i) => ({
  id: i,
  col1: `row ${i} col 1`,
  col2: `row ${i} col 2`,
}));

  // Callback used on pagination event
  const onPageChange = (page, sizePerPage) => {};
  // Callback used on size per page change event
  const onSizePerPageChange = (sizePerPage, page) => {};

<Table 
  title={'Table With Pagination'}
  data={aLotOfData}
  onPageChange={onPageChange}
  onSizePerPageChange={onSizePerPageChange}
  columns={[
    { dataField: 'col1', text: 'column 1', sort: true }, 
    { dataField: 'col2', text: 'column 2' },
  ]}
/>
```
### row event handlers and styling

```jsx
const clickRow =  (e, row, rowIndex) => {
  alert(`clicked on row: ${JSON.stringify(row, null, 2)}`);
};

const rowClasses = (row, rowIndex) => {
  return rowIndex === 1 ? 'selected' : null;
}

<Table 
  title={'Table With Row Events'}
  data={[
    { id: 'row1Id', col1: 'row 1 col 1', col2: 'row 1 col 2' },
    { id: 'row2Id', col1: 'row 2 col 1', col2: 'row 2 col 2' },
    { id: 'row3Id', col1: 'row 3 col 1', col2: 'row 3 col 2' },
  ]}
  columns={[
    { dataField: 'col1', text: 'column 1', sort: true }, 
    { dataField: 'col2', text: 'column 2' },
  ]}
  rowEvents={{ onClick: clickRow }}
  rowClasses={rowClasses}
/>
```

### custom column rendering

```jsx
import { A } from 'state-template';

const rowFormat = (cell, row, rowIndex) => (
  <A onClick={() => console.log('clicked row button', cell, row, rowIndex)} text={`link ${rowIndex}`} />
);

<Table 
  hideSearch
  title={'Table With Custom Rendering'}
  data={[
    { id: 'row1Id', col1: 'row 1 col 1', col2: 'row 1 col 2' },
    { id: 'row2Id', col1: 'row 2 col 1', col2: 'row 2 col 2' },
  ]}
  columns={[
    { dataField: 'col1', text: 'column 1', sort: true }, 
    { dataField: 'col2', text: 'column 2', formatter: rowFormat },
  ]}
/>
```

### with menu

```jsx
import { Button, ButtonRow } from 'state-template';

const menu = (
  <ButtonRow>
    <Button text={'new'} onClick={() => {}} variant={'primary'} />
    <Button text={'edit'} onClick={() => {}} variant={'default'} />
  </ButtonRow>
);

<Table 
  data={[
    { id: 'row1Id', col1: 'row 1 col 1', col2: 'row 1 col 2' },
    { id: 'row2Id', col1: 'row 2 col 1', col2: 'row 2 col 2' },
  ]}
  columns={[
    { dataField: 'col1', text: 'column 1', sort: true }, 
    { dataField: 'col2', text: 'column 2' },
  ]}
  menu={menu}
/>
```

### with menu and title, no search

```jsx
import { Button, ButtonRow } from 'state-template';

const menu = (
  <ButtonRow>
    <Button text={'new'} onClick={() => {}} variant={'primary'} />
    <Button text={'edit'} onClick={() => {}} variant={'default'} />
  </ButtonRow>
);

<Table 
  hideSearch
  title={'Table With No Search'}
  data={[
    { id: 'row1Id', col1: 'row 1 col 1', col2: 'row 1 col 2' },
    { id: 'row2Id', col1: 'row 2 col 1', col2: 'row 2 col 2' },
  ]}
  columns={[
    { dataField: 'col1', text: 'column 1', sort: true }, 
    { dataField: 'col2', text: 'column 2' },
  ]}
  menu={menu}
/>
```
